import React, { useState, useEffect, useImperativeHandle } from 'react'
import { Space, Table, Modal, message, Popconfirm, Button } from 'antd'
import DatasourceData from '../component/DatasourceData'
import DatasourceEdit from '../component/DatasourceEdit'
import {
  getDataSourceList,
  delDataSource,
  getTableNamesList,
  getDataSourceInfo,
} from '../services'

export default function DatasourceManage({ onRef }) {
  // 查看数据源弹窗是否可可视
  const [dataModalVisible, setDataModalVisible] = useState(false)
  const [dsModalVisible, setDsModalVisible] = useState(false)

  // 初始化数据源列表数据 constructor
  const [dataSourceList, setDataSourceList] = useState([])

  // 表格Loading状态
  const [dataLoading, setDataLoading] = useState(false)

  // 表名
  const [tableNames, setTableNames] = useState([])

  // 选中数据源
  const [datasourceId, setDatasourceId] = useState(null)

  // 查看是否可点击
  const [tableWatcher, setTableWatcher] = useState(false)

  const [currentDsId, setCurrentDsId] = useState(null)
  const [currentDsInfo, setCurrentDsInfo] = useState(null)

  // 获取列表数据
  const fetchDataSourceList = async () => {
    setDataLoading(true)
    const { data } = await getDataSourceList()
    if (data.code === 200) {
      const list = data.data.items.map(item => {
        return {
          key: item.id,
          datasource_name: item.datasource_name,
          datasource: item.datasource,
          tag: item.tag,
          comments: item.comments,
          create_time: item.create_time,
          update_time: item.update_time,
        }
      })
      setDataSourceList(list)
    }
    setDataLoading(false)
  }

  // 暴露更新列表方法
  useImperativeHandle(onRef, () => {
    return {
      updateSourceList: fetchDataSourceList,
    }
  })

  // 删除确认
  const DeleteConfirm = async key => {
    const { data } = await delDataSource(key)
    if (data.code === 200) {
      message.success('删除成功')
      fetchDataSourceList()
    } else {
      message.success('删除失败')
    }
  }

  //获取数据源表名
  const fetchTableNames = async key => {
    setTableWatcher(true)
    const { data } = await getTableNamesList(key)
    if (data.code === 200) {
      setDatasourceId(key)
      setTableNames(data.data)
      setDataModalVisible(true)
    } else {
      message.error(data.msg)
    }
    setTableWatcher(false)
  }

  // componentDidMount
  useEffect(() => {
    fetchDataSourceList()
  }, [])

  // 格式化事件
  const formatTime = time => {
    const date = new Date(Number(time) * 1000)
    const YY = date.getFullYear()
    const MM =
      date.getMonth() + 1 < 10
        ? '0' + (date.getMonth() + 1)
        : date.getMonth() + 1
    const DD = date.getDate() < 10 ? '0' + date.getDate() : date.getDate()
    const hh = date.getHours() < 10 ? '0' + date.getHours() : date.getHours()
    const mm =
      date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes()
    const ss =
      date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds()
    return YY + '-' + MM + '-' + DD + ' ' + hh + ':' + mm + ':' + ss
  }

  // 编辑数据
  const editData = async id => {
    const { data } = await getDataSourceInfo(id)
    if (data.code === 200) {
      setCurrentDsId(id)
      setCurrentDsInfo(data.data)
      setDsModalVisible(true)
    } else {
      message.error(data.msg)
    }
  }

  // 表格项
  const columns = [
    {
      title: '数据源名称',
      dataIndex: 'datasource_name',
      key: 'datasource_name',
    },
    {
      title: '数据源类型',
      dataIndex: 'datasource',
      key: 'datasource',
    },
    {
      title: '标签',
      dataIndex: 'tag',
      key: 'tag',
      render: text => <span style={{ color: '#1890ff' }}>{text}</span>,
    },
    {
      title: '详情',
      key: 'comments',
      dataIndex: 'comments',
    },
    {
      title: '创建时间',
      key: 'create_time',
      dataIndex: 'create_time',
      render: time => formatTime(time),
    },
    {
      title: '更新时间',
      key: 'update_time',
      dataIndex: 'update_time',
      render: time => formatTime(time),
    },
    {
      title: '操作',
      key: 'operation',
      render: (_, record) => (
        <Space size="middle">
          <Button
            onClick={() => {
              fetchTableNames(record.key)
            }}
            type="link"
            disabled={tableWatcher}>
            查看
          </Button>
          <Button
            type="link"
            onClick={() => {
              editData(record.key)
            }}>
            编辑
          </Button>
          <Popconfirm
            title="确认删除？"
            okText="确认"
            cancelText="取消"
            onConfirm={() => DeleteConfirm(record.key)}>
            <Button type="link">删除</Button>
          </Popconfirm>
        </Space>
      ),
    },
  ]
  // 查看数据源弹窗取消
  const handledataModalCancel = () => {
    setDataModalVisible(false)
  }

  const handledsModalCancel = () => {
    setDsModalVisible(false)
  }

  // 查看数据源弹窗数据
  const dataModal = (
    <Modal
      title="数据表预览"
      visible={dataModalVisible}
      onCancel={handledataModalCancel}
      footer={null}
      destroyOnClose={true}
      width={'80%'}
      bodyStyle={{ paddingTop: 0 }}>
      <DatasourceData data={tableNames} datasourceId={datasourceId} />
    </Modal>
  )

  const dsEditModal = (
    <Modal
      title="编辑数据源"
      visible={dsModalVisible}
      onCancel={handledsModalCancel}
      footer={null}
      destroyOnClose={true}
      width={'60%'}>
      <DatasourceEdit
        ds_id={currentDsId}
        ds_info={currentDsInfo}
        updateDataSource={fetchDataSourceList}
        setDsModalVisible={val => setDsModalVisible(val)}
      />
    </Modal>
  )

  // 最终渲染
  return (
    <>
      <Table
        columns={columns}
        dataSource={dataSourceList}
        bordered
        loading={dataLoading}
      />
      {dataModal}
      {dsEditModal}
    </>
  )
}
